#!/usr/bin/perl -w

$nbh      = 1; # Black hole
$nplanet  = 50;
$nstar    = 10;
$ncomet   = 200;
$nobject  = $nbh+$nstar+$nplanet+$ncomet;
$mplanet  = 1;
$mstar    = 1e5;
$mbh      = 1e9;
$mcomet   = 0.01;
$var      = 0.8;
$univx    = 1e6; # Distance units is Earth-Sun distances
$univy    = 1e6;
$univz    = 1e6;
$vearth   = 29.783 / 149597870;
$temp     = 0;
$dt       = 100; # Time units is seconds
$nsteps   = 100000; 
$seed     = 1993;
srand($seed);

sub mk_mdp {
  my $pbc = shift;
  my $mdp = "grompp.mdp";
  open(MDP,">$mdp") || die("Opening $mdp");
  printf(MDP "; $mdp generated by gd.pl\n");
  printf(MDP "; (c) 2008 David van der Spoel\n");
  printf(MDP "nsteps     = %d\n",$nsteps);
  printf(MDP "dt         = %g\n",$dt);
  my $rc = 0;
  if ( $pbc ) {
    printf(MDP "comm_mode  = linear\n");
    $rc = $univz*0.4;
    printf(MDP "nstlist    = 5\n");
    printf(MDP "nstype     = grid\n");
    printf(MDP "pbc        = xyz\n");
  }
  else {
    printf(MDP "comm_mode  = angular\n");
    $rc = 0;
    printf(MDP "nstlist    = 5\n");
    printf(MDP "nstype     = simple\n");
    printf(MDP "pbc        = no\n");
  }
  printf(MDP "rcoulomb   = %g\n",$rc);
  printf(MDP "rvdw       = %g\n",$rc);
  printf(MDP "rlist      = %g\n",$rc);
  printf(MDP "integrator = md\n");
  printf(MDP "epsilon_r  = -900347043\n");
  printf(MDP "nstxout    = 10\n");
  printf(MDP "nstvout    = 0\n");
  printf(MDP "nstfout    = 0\n");
  printf(MDP "nstxtcout  = 0\n");
  printf(MDP "nstlog     = 10\n");
  printf(MDP "nstenergy  = 10\n");
  if ($temp > 0) {
    printf(MDP "gen_vel    = yes\n");
    printf(MDP "gen_temp   = %g\n",$temp);
    printf(MDP "tcoupl     = Berendsen\n");
    printf(MDP "tc-grps    = System\n");
    printf(MDP "ref-t      = %g\n",$temp);
    printf(MDP "tau-t      = %g\n",100*$dt);
  }
  close MDP;
}

sub mk_mass {
  my $aver = shift;
  
  my $vv = $aver*(1 + $var*(2.0*rand() -1.0));
  
  return $vv;
}

sub g96_line {
  my $aid = shift;
  my $anm = shift;
  my $rnm = shift;
  my $rid = shift;
  my $x = shift;
  my $y = shift;
  my $z = shift;
  
  return sprintf("%5d %5s %5s %5d   %13.6e  %13.6e  %13.6e\n",
		 $aid,$anm,$rnm,$rid,$x,$y,$z);
}

sub mk_coords {
  my @coord = ();
  my $r2min = $univx/5;
  my $i = my $j = 0;
  my $x = my $y = my $z = 0;
  for($i=0; ($i < $nobject); ) {
    if (($nbh > 0) && ($i == 0)) {
      $x = $univx/2;
      $y = $univy/2;
      $z = $univz/2;
    }
    else {
      $x = rand($univx);
      $y = rand($univy);
      $z = rand($univz);
    }
    my $r2 = 2*$r2min;
    for($j=0; ($j < $i) && ($r2 >= $r2min); $j++) {
      $r2 = (($coord[3*$j] - $x)**2 + ($coord[3*$j+1] - $y)**2 +
	     ($coord[3*$j+2] - $z)**2);
    }
    if ($j == $i) {
      $coord[3*$i]   = $x;
      $coord[3*$i+1] = $y;
      $coord[3*$i+2] = $z;
      $i++;
    }
  }
  return @coord;
}

sub mk_veloc {
  my @veloc = ();
  my $r2min = $univz/10;
  my $i = my $j = 0;
  my $x = my $y = my $z = 0;
  for($i=0; ($i < $nobject); ) {
    if (($nbh > 0) && ($i == 0)) {
      $x = $y = $z = 0;
    } 
    else {
      $x = $vearth*(2*rand()-1);
      $y = $vearth*(2*rand()-1);
      $z = $vearth*(2*rand()-1);
    }
    $veloc[3*$i]   = $x;
    $veloc[3*$i+1] = $y;
    $veloc[3*$i+2] = $z;
    $i++;
  }
  return @veloc;
}

sub mk_tx {
  my $top = shift;
  my $g96 = shift;
  
  open(G96,">$g96") || die("Opening $g96");
  printf(G96 "TITLE\nGalactic dynamics - David van der Spoel\nEND\n");
  printf(G96 "POSITION\n");
  printf("Writing $top and $g96\n");
  
  open(TOP,"> topol.top") || die("Opening topol.top");
  printf(TOP "[ defaults ]\n");
  printf(TOP " 1 1 no 1 1\n\n");
  printf(TOP "[ atomtypes ]\n");
  printf(TOP "CO CO 1 1 0 A 0 0\n");
  printf(TOP "[ molecule_type ]\n");
  printf(TOP "Galaxy 0\n\n");
  printf(TOP "[ atoms ]\n");

  my @coord = mk_coords();
  my @veloc = mk_veloc();
  
  my $k  = 1;
  my $m = my $i = my $j = my $c0 = 0;
  my $anm = ""; 
  @aname = ();
  for($i=1; ($i<= $nbh); $i++,$k++) {
    $m = 1e6*$mstar;
    $anm = "N";
    $aname[$k] = $anm;
    printf(TOP "%d  CO %d %s %s %d $m  $m\n",$k,$k,$anm,$anm,$k); 
    printf(G96 "%s",g96_line($k,$anm,$anm,$k,
			     $coord[$c0],$coord[$c0 + 1],$coord[$c0 + 2]));
    $c0 += 3;
  }
  for($i=1; ($i<= $nstar); $i++,$k++) {
    $m = mk_mass($mstar);
    $anm = "S";
    $aname[$k] = $anm;
    printf(TOP "%d  CO %d %s %s %d $m  $m\n",$k,$k,$anm,$anm,$k); 
    printf(G96 "%s",g96_line($k,$anm,$anm,$k,
			     $coord[$c0],$coord[$c0 + 1],$coord[$c0 + 2]));
    $c0 += 3;
  }
  for($i=1; ($i<= $nplanet); $i++,$k++) {
    $m = mk_mass($mplanet);
    $anm = "P";
    $aname[$k] = $anm;
    printf(TOP "%d  CO %d %s %s %d $m  $m\n",$k,$k,$anm,$anm,$k); 
    printf(G96 "%s",g96_line($k,$anm,$anm,$k,
			     $coord[$c0],$coord[$c0 + 1],$coord[$c0 + 2]));
    $c0 += 3;
  }
  for($i=1; ($i<= $ncomet); $i++,$k++) {
    $m = mk_mass($mcomet);
    $anm = "C";
    $aname[$k] = $anm;
    printf(TOP "%d  CO %d %s %s %d $m  $m\n",$k,$k,$anm,$anm,$k); 
    printf(G96 "%s",g96_line($k,$anm,$anm,$k,
			     $coord[$c0],$coord[$c0 + 1],$coord[$c0 + 2]));
    $c0 += 3;
  }
 
  printf(TOP "[ system ]\n");
  printf(TOP "My Little Galaxy\n\n");
  printf(TOP "[ molecules ]\n");
  printf(TOP "Galaxy 1\n");
  
  close TOP;
  printf(G96 "END\nVELOCITY\n");
  $c0 = 0;
  for($i=1; ($i<= $nobject); $i++) {
    printf(G96 "%s",g96_line($i,$aname[$i],$aname[$i],$i,
			     $veloc[$c0],$veloc[$c0+1],$veloc[$c0+2]));
    $c0 += 3;
  }
  printf(G96 "END\nBOX\n  %13.6e  %13.6e  %13.6e\nEND\n",
	 $univx,$univy,$univz);
  
  close G96;
}

# Main code!
mk_mdp(1);
$tab = "gravity.xvg";
mk_tx("topol.top","conf.g96");

$tpr = "topol.tpr";
if ( -f $tpr ) {
  unlink $tpr;
}
$ENV{"GALACTIC_DYNAMICS"} = 1;
system("grompp -v -o $tpr");
if ( -f $tpr ) {
  system("mpirun -c 1 mdrun -v -pd -c confout.g96");
  system("ngmx -s $tpr");
}
else {
  die("grompp failed");
}
